package net.p0f.samples.rhpam_fuse_integration;

import java.util.HashMap;
import java.util.Map;

import org.kie.api.KieServices;
import org.kie.api.builder.KieScanner;
import org.kie.api.builder.ReleaseId;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import net.p0f.samples.rhpam_fuse_integration.shared_dom.Award;
import net.p0f.samples.rhpam_fuse_integration.shared_dom.Customer;

// To run this class, you will need to have created a RHPAM project called
// pam-simple-process (or import one from a Git repository, feel free to
// fork the one I shared), then make it available to Maven somehow (either
// by adding internal artifact repository from RHPAM to settings.xml or by
// building the KJAR and installing it; note that in dev mode, RHPAM will
// install artifacts to local Maven repository of its owning user anyway).
//
// NOTE: both this project and the RHPAM one depend on shared-data-model.
public class UseRemoteKieArtifact {
	public static void main(String... args) {
		// Create a reference to GAV item (need kie-ci dependency for this!)
		//
		// NOTE: with the packaging of kjar, LATEST or RELEASE pseudo-versions can't be used.
		ReleaseId artifact = KieServices.Factory.get().newReleaseId("net.p0f.samples.rhpam-fuse-integration", "pam-simple-process", "1.0.0-SNAPSHOT");

		// Create a generic container referencing that release:
		KieContainer c = KieServices.Factory.get().newKieContainer(artifact);

		// It is possible to automatically update the above periodically (10 minutes in
		// this case):
		KieScanner ks = KieServices.Factory.get().newKieScanner(c);
		ks.start(600000);

		// Create a new session containing artifacts from the remote repository.
		// NOTE: ANY SUBSEQUENT KB UPDATES ARE NOT VISIBLE TO THIS SESSION!
		KieSession s = c.newKieSession();

		Customer c1 = new Customer();
		c1.setName("John Doe");
		c1.setAge(27);
		c1.setLoyaltyLevel(Customer.GOLD);
		c1.setLoyaltyAge(10);
		c1.setMilesFlown(15000);

		Award a1 = new Award();
		a1.setCarrier(c1);

		Map<String, Object> p = new HashMap<String, Object>();
		p.put("customer", c1);
		p.put("award", a1);

		s.startProcess("pam-simple-process.SayHello", p);

		s.fireAllRules();

		s.dispose();
	}
}
