package net.p0f.samples.rhpam_fuse_integration.processes;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.QueryResults;
import org.kie.api.runtime.rule.QueryResultsRow;

import net.p0f.samples.rhpam_fuse_integration.shared_dom.Award;
import net.p0f.samples.rhpam_fuse_integration.shared_dom.Customer;

public class ProcessWithListSubProcess {
	public static void main(String... args) {
		KieContainer c = KieServices.Factory.get().getKieClasspathContainer();

		// Instantiate the ruleflowParams KB and its default session.
		KieSession s = c.getKieBase("ruleflowParams").newKieSession();

		Customer c1 = new Customer();
		c1.setName("John Doe");
		c1.setAge(27);
		c1.setLoyaltyLevel(Customer.GOLD);
		c1.setLoyaltyAge(10);
		c1.setMilesFlown(15000);

		Customer c2 = new Customer();
		c2.setName("Jane Doe");
		c2.setAge(35);
		c2.setLoyaltyLevel(Customer.PLATINUM);
		c2.setMilesFlown(45000);
		c2.setLoyaltyAge(10);

		Award a1 = new Award();
		a1.setCarrier(c1);
		Award a2 = new Award();
		a2.setCarrier(c2);

		// Wrapped types don't match the rule conditions, but see the process model!
		List<Customer> customers = new ArrayList<>();
		List<Award> awards = new ArrayList<>();

		customers.add(c1);
		customers.add(c2);

		awards.add(a1);
		awards.add(a2);

		Map<String, Object> p = new HashMap<>();
		p.put("customers", customers);
		p.put("awards", awards);

		// In this process model, we're using a hybrid approach: there are two embedded
		// multi-instance sub-processes, each of which is responsible for iterating over
		// one of the list parameters and registering its contents as facts.
		// This also has an important consequence how the results are processed in the
		// rules, and how we process the outcome (congratulations "task" is another
		// multi-instance sub-process).
		// As said, it is ugly and non-portable though, because of the script tasks.
		s.startProcess("ruleflow.RulesWithListSubProc", p);
		s.fireAllRules();

		System.out.println("******** Examining Working Memory ********");

		// One benefit, if anything, is that the results are available both as facts in
		// working memory...
		QueryResults qr = s.getQueryResults("Get Awards");
		for (QueryResultsRow qrr : qr) {
			Award award = (Award) qrr.get("award");
			System.out.println("Found award: " + award.toString());
		}

		System.out.println("******** Examining Process Parameters ********");

		// ...and thanks to pass-by-reference, they also get updated in the respective
		// process parameter lists.
		System.out.println("Customers: " + customers);
		System.out.println("Awards: " + awards);

		s.dispose();

	}
}
