package net.p0f.samples.rhpam_fuse_integration.processes;

import java.util.HashMap;
import java.util.Map;

import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.QueryResults;
import org.kie.api.runtime.rule.QueryResultsRow;

import net.p0f.samples.rhpam_fuse_integration.shared_dom.Award;
import net.p0f.samples.rhpam_fuse_integration.shared_dom.Customer;

public class ProcessWithParams {
	public static void main(String... args) {
		KieContainer c = KieServices.Factory.get().getKieClasspathContainer();

		// Instantiate the ruleflow KB and its default session.
		KieSession s = c.getKieBase("ruleflow").newKieSession();

		Customer c1 = new Customer();
		c1.setName("Jane Doe");
		c1.setAge(35);
		// Deliberately not setting LL
		// c1.setLoyaltyLevel(Customer.PLATINUM);
		c1.setMilesFlown(45000);
		c1.setLoyaltyAge(10);

		Award a1 = new Award();

		// Option 2: Pass facts as parameters.
		Map<String, Object> params = new HashMap<>();
		params.put("customer", c1);
		params.put("award", a1);

		// Process ID does not have to follow package naming convention.
		//
		// HOWEVER: While rules are loaded regardless of their location in the
		// filesystem, and the "package" keyword decides which KBs will see them or not,
		// process models MUST be in the correct location in the filesystem (iow, if the
		// directory path does not equal the package of the process, including
		// underscores in package names, the KieContainer will not load it into the KB).
		s.startProcess("ruleflow.RulesRulesRules", params);
		s.fireAllRules();

		System.out.println("******** Examining Working Memory ********");

		// See below.
		QueryResults qr = s.getQueryResults("Get Awards");
		for (QueryResultsRow qrr : qr) {
			Award award = (Award) qrr.get("award");
			System.out.println("Found award: " + award.toString());
		}

		System.out.println("******** Examining Process Parameters ********");

		// Note that with the way rules are written in this example, an existing process
		// parameter of type Award does NOT get modified - one has to write rules
		// differently for that, see the ruleflow_params package examples for that.
		System.out.println("Customer: " + c1);
		System.out.println("Award: " + a1);

		s.dispose();
	}
}
